@@ -185,6 +185,14 @@ def selenium():
         return None
     return selenium
 
+def selenium_webdriver():
+    try:    from selenium import webdriver as selenium_webdriver
+    except Exception as err:
+        print('  IMPORT FAIL |selenium_webdriver|, err:|{}|'.format(err))
+        return None
+    return selenium_webdriver
+
+
 
 #-----------------------------------------------------------------
 # FTP
