@@ -22,86 +22,100 @@ except:
 os = lib.os()
 pd = lib.pandas()
 pyodbc = lib.pyodbc()
-lite =   lib.sqlite3()
+lite = lib.sqlite3()
 
 
-
-#---------------------------------------------------------------
-#------------- CLASS DB management -----------------------------
-#---------------------------------------------------------------
+# ---------------------------------------------------------------
+# ------------- CLASS DB management -----------------------------
+# ---------------------------------------------------------------
 @oth.dec_singletonsClass
 class c_db_lite:
     """ This class allows you to manage simple lite database"""
+
     def __init__(self):
         self.str_pathDb = None
         self.cnxn = None
+
     def definePath(self, str_pathDb):
         self.str_pathDb = str_pathDb
+
     def connect(self):
         cnxn = lite.connect(self.str_pathDb)
         self.cnxn = cnxn
+
     def getDataframe(self, str_req):
         df_req = pd.read_sql_query(str_req, self.cnxn)
         self.df_req = df_req
         return df_req
+
     def closeConnection(self):
-        try:    self.cnxn.close()
-        except: pass
+        try:
+            self.cnxn.close()
+        except:
+            pass
         self.cnxn = None
+
     def __del__(self):
         self.closeConnection()
 
 
-
 class c_db_sqlServer:
     def __init__(self):
         self.__dict_Connect = {}
         self.__bl_AlertIfEmptyReq = True
         self.__timeout = 100
         self.__request = 'Request not defined'
-        self.cnxn =         None
-        self.cursor =       None
-        self.__server =     None
-        self.__database =   None
-        self.__uid =        None
-        self.__pwd =        None
+        self.cnxn = None
+        self.cursor = None
+        self.__server = None
+        self.__database = None
+        self.__uid = None
+        self.__pwd = None
+
     def defineCredentials(self, **d_param):
         if 'server' in d_param:
-            self.__server =     d_param['server']
+            self.__server = d_param['server']
         if 'database' in d_param:
-            self.__database =   d_param['database']
+            self.__database = d_param['database']
         if 'uid' in d_param:
-            self.__uid =        d_param['uid']
+            self.__uid = d_param['uid']
         if 'pwd' in d_param:
-            self.__pwd =        d_param['pwd']
+            self.__pwd = d_param['pwd']
         if 'timeout' in d_param:
             self.__timeout = d_param['timeout']
         if 'bl_AlertIfEmptyReq' in d_param:
             self.__bl_AlertIfEmptyReq = d_param['bl_AlertIfEmptyReq']
+
     @property
     def server(self):
         return self.__server
+
     @property
     def database(self):
         return self.__database
+
     @property
     def uid(self):
         return self.__uid
+
     @property
     def pwd(self):
         return self.__pwd
+
     @property
     def timeout(self):
         return self.__timeout
+
     @property
     def request(self):
         return self.__request
+
     @request.setter
-    def request(self, str_request = ''):
+    def request(self, str_request=''):
         if str_request != '':
             self.__request = str_request
 
-    def __str__(self, message = ''):
+    def __str__(self, message=''):
         str_msg = '--------------------------------------\n'
         str_msg += '  - |{}|, |{}|, |{}| \n'.format(self.__server, self.__database, self.__uid)
         str_msg += '  - |{}| \n'.format(self.__request)
@@ -110,18 +124,18 @@ class c_db_sqlServer:
         print(str_msg)
         return str_msg
 
-    def connect(self):                  # db_sqlConnectCursor
+    def connect(self):  # db_sqlConnectCursor
         i_cnxReturn = 0
         t_keyConnect = (self.__server, self.__database, self.__uid)
         if t_keyConnect not in self.__dict_Connect:
-            #----------------------------------------------------------------------------------
+            # ----------------------------------------------------------------------------------
             # NEW connexion
             try:
                 self.cnxn = pyodbc.connect('DRIVER={SQL Server};SERVER=' + self.__server +
                                            ';DATABASE=' + self.__database +
                                            ';UID=' + self.__uid +
                                            ';PWD=' + self.__pwd,
-                                           timeout = self.__timeout)
+                                           timeout=self.__timeout)
                 # trusted_connection=YES;  for Windows Authentification
                 # Save the connexion in a dico with KEY = ID in tuple0
             except Exception as err:
@@ -145,30 +159,33 @@ class c_db_sqlServer:
             i_cnxReturn = 1
             return i_cnxReturn
         # CURSOR
-        try:    self.cursor = self.cnxn.cursor()
+        try:
+            self.cursor = self.cnxn.cursor()
         except Exception as err:
             self.__str__(' ERROR: Your db CURSOR is not working || {} || {}'.format(err, str(i_cnxReturn)))
             raise
         # END
         return i_cnxReturn
 
-    def availablePyodbcDrivers(self):       # db_seeDriversAvailable
+    def availablePyodbcDrivers(self):  # db_seeDriversAvailable
         self.__drivers = pyodbc.drivers()
         return self.__drivers
 
-    def executeReq(self):                   # db_Execute
-        try:    self.cursor.execute(self.__request)
+    def executeReq(self):  # db_Execute
+        try:
+            self.cursor.execute(self.__request)
         except Exception as err:
             self.__str__(' ERROR: executeReq is not working || {}'.format(err))
             raise
 
-    def commit(self):                       # db_Commit
-        try:    self.cnxn.commit()
+    def commit(self):  # db_Commit
+        try:
+            self.cnxn.commit()
         except Exception as err:
             self.__str__(' ERROR: commit is not working || {}'.format(err))
             raise
 
-    def getDataframe(self):                 # getDataFrame_fReq
+    def getDataframe(self):  # getDataFrame_fReq
         self.df_result = False
         try:
             self.df_result = pd.read_sql(self.__request, self.cnxn)
@@ -184,17 +201,17 @@ class c_db_sqlServer:
     def getDataframe_multipleReq(self):
         self.df_result = False
         try:
-            l_resultSet =   self.cursor.fetchall()
-            df_resultSet =  pd.DataFrame.from_records(l_resultSet)
-            while ( self.cursor.nextset() ):
-                l_resultSet =       self.cursor.fetchall()
+            l_resultSet = self.cursor.fetchall()
+            df_resultSet = pd.DataFrame.from_records(l_resultSet)
+            while (self.cursor.nextset()):
+                l_resultSet = self.cursor.fetchall()
                 df_resultSet_Suiv = pd.DataFrame.from_records(l_resultSet)
-                df_resultSet =      self.fDf_Concat(df_resultSet, df_resultSet_Suiv)
+                df_resultSet = self.fDf_Concat(df_resultSet, df_resultSet_Suiv)
             # Final Result
-            self.df_result =      df_resultSet
+            self.df_result = df_resultSet
             # Message if empty
             if self.__bl_AlertIfEmptyReq is True:
-                if self.df_result.empty or self.df_result.dropna(how = 'all').empty:
+                if self.df_result.empty or self.df_result.dropna(how='all').empty:
                     self.__str__(' EMPTY: getDataframe_multipleReq is empty')
         except Exception as err:
             self.__str__(' ERROR: getDataframe_multipleReq is not working || {}'.format(err))
@@ -220,15 +237,22 @@ class c_db_sqlServer:
         try:
             self.__dict_Connect = {}
             self.__str__(' INFO: Your db connexion is closing !')
-        except: pass
+        except:
+            pass
         try:
             self.cursor.close()
             del self.cursor
-        except: pass
-        try:    self.cnxn.close()
-        except: pass
-        try:    self.cnxn = None
-        except: pass
+        except:
+            pass
+        try:
+            self.cnxn.close()
+        except:
+            pass
+        try:
+            self.cnxn = None
+        except:
+            pass
+
     def __del__(self):
         self.closeConnection()
 
@@ -236,6 +260,7 @@ class c_db_sqlServer:
 @oth.dec_singletonsClass
 class c_db_sqlServ_single(c_db_sqlServer):
     """ This inehrit from c_db_sqlServer and is a singleton"""
+
     def __init__(self):
         super().__init__()
 
@@ -243,45 +268,50 @@ class c_db_sqlServ_single(c_db_sqlServer):
 class c_db_dfCredentials(c_db_sqlServer):
     """ This inehrit from c_db_sqlServer and is a singleton
     Manage the Credentials in a dataframe form"""
+
     def __init__(self):
         super().__init__()
         self.df_UID = None
+
     def dataframeCredentials(self, df_UID):
         self.df_UID = df_UID
         # Get the columns of the UID given the order of columns are: Server, database, UID, Password
         l_columns = df_UID.columns
-        self.serverColName =    l_columns[0]
-        self.databaseColName =  l_columns[1]
-        self.uidColName =       l_columns[2]
-        self.pwdColName =       l_columns[3]
+        self.serverColName = l_columns[0]
+        self.databaseColName = l_columns[1]
+        self.uidColName = l_columns[2]
+        self.pwdColName = l_columns[3]
         try:
-            str_server =     df_UID[self.serverColName].values[0]
-            str_database =   df_UID[self.databaseColName].values[0]
-            str_uid =        df_UID[self.uidColName].values[0]
-            str_pwd =        df_UID[self.pwdColName].values[0]
+            str_server = df_UID[self.serverColName].values[0]
+            str_database = df_UID[self.databaseColName].values[0]
+            str_uid = df_UID[self.uidColName].values[0]
+            str_pwd = df_UID[self.pwdColName].values[0]
         except:
             self.serverColName = 'Server'
             self.databaseColName = 'Database'
             self.uidColName = 'Uid'
             self.pwdColName = 'Password'
-            str_server =    df_UID[self.serverColName].values[0]
-            str_database =  df_UID[self.databaseColName].values[0]
-            str_uid =       df_UID[self.uidColName].values[0]
-            str_pwd =       df_UID[self.pwdColName].values[0]
+            str_server = df_UID[self.serverColName].values[0]
+            str_database = df_UID[self.databaseColName].values[0]
+            str_uid = df_UID[self.uidColName].values[0]
+            str_pwd = df_UID[self.pwdColName].values[0]
         # Keep in memory the last working server
         self.__last_server = str_server
         # Set the value by the original Class
-        d_param = dict(server=str_server, database=str_database, uid=str_uid,pwd=str_pwd)
+        d_param = dict(server=str_server, database=str_database, uid=str_uid, pwd=str_pwd)
         self.defineCredentials(**d_param)
-    def change_server(self, str_server = ''):
+
+    def change_server(self, str_server=''):
         if not str_server == '':
             df_UID = self.df_UID
             try:
-                str_database =   df_UID.loc[df_UID[self.serverColName] == str_server, self.databaseColName].values[0]
-                str_uid =        df_UID.loc[df_UID[self.serverColName] == str_server, self.uidColName].values[0]
-                str_pwd =        df_UID.loc[df_UID[self.serverColName] == str_server, self.pwdColName].values[0]
+                str_database = df_UID.loc[df_UID[self.serverColName] == str_server, self.databaseColName].values[0]
+                str_uid = df_UID.loc[df_UID[self.serverColName] == str_server, self.uidColName].values[0]
+                str_pwd = df_UID.loc[df_UID[self.serverColName] == str_server, self.pwdColName].values[0]
             except Exception as err:
-                print('ERROR in change_server: we could not find the server: |{}| in the Dataframe provided \n'.format(str_server))
+                print('ERROR in change_server: we could not find the server: |{}| in the Dataframe provided \n'.format(
+                    str_server))
+                print(err)
                 print(df_UID)
                 return False
             # Keep in memory the last working server
@@ -290,7 +320,8 @@ class c_db_dfCredentials(c_db_sqlServer):
             d_param = dict(server=str_server, database=str_database, uid=str_uid, pwd=str_pwd)
             self.defineCredentials(**d_param)
         return True
-    def change_database(self, str_database = ''):
+
+    def change_database(self, str_database=''):
         if not str_database == '':
             df_UID = self.df_UID
             str_server = self.__last_server
@@ -305,6 +336,7 @@ class c_db_dfCredentials(c_db_sqlServer):
 @oth.dec_singletonsClass
 class c_db_dataframeCred(c_db_dfCredentials):
     """ This inehrit from c_db_dfCredentials + is a singleton"""
+
     def __init__(self):
         super().__init__()
 
@@ -314,29 +346,35 @@ class c_db_withLog(c_db_dfCredentials):
     """ This inehrit from c_db_dfCredentials + is a singleton
     Allow you to manage some Log request inside a particular server and database
     without changing the connection of the other requests"""
+
     def __init__(self):
         super().__init__()
         self.__server_default = None
+        self.bl_useLog = False
+
     def dataframeCredentials(self, df_UID):
         super().dataframeCredentials(df_UID)
-        self.__server_default =     self.server
-        self.__database_default =   self.database
-        self.__uid_default =        self.uid
-        self.__pwd_default =        self.pwd
-    def change_server(self, str_server = ''):
+        self.__server_default = self.server
+        self.__database_default = self.database
+        self.__uid_default = self.uid
+        self.__pwd_default = self.pwd
+
+    def change_server(self, str_server=''):
         super().change_server(str_server)
         if not str_server == '':
-            self.__server_default =     self.server
-            self.__database_default =   self.database
-            self.__uid_default =        self.uid
-            self.__pwd_default =        self.pwd
-    def define_Log_Cred(self, str_serverForLog = None, str_databaseForLog = None):
+            self.__server_default = self.server
+            self.__database_default = self.database
+            self.__uid_default = self.uid
+            self.__pwd_default = self.pwd
+
+    def define_Log_Cred(self, str_serverForLog=None, str_databaseForLog=None):
         df_UID = self.df_UID
-        self.server_Log =       str_serverForLog
-        self.db_Log =           str_databaseForLog
-        self.__uid_Log =        df_UID.loc[df_UID[self.serverColName] == self.server_Log, self.uidColName].values[0]
-        self.__pwd_Log =        df_UID.loc[df_UID[self.serverColName] == self.server_Log, self.pwdColName].values[0]
-    def executeLog(self, str_logExec = ''):
+        self.server_Log = str_serverForLog
+        self.db_Log = str_databaseForLog
+        self.__uid_Log = df_UID.loc[df_UID[self.serverColName] == self.server_Log, self.uidColName].values[0]
+        self.__pwd_Log = df_UID.loc[df_UID[self.serverColName] == self.server_Log, self.pwdColName].values[0]
+
+    def executeLog(self, str_logExec=''):
         d_pLog = dict(server=self.server_Log, database=self.db_Log, uid=self.__uid_Log, pwd=self.__pwd_Log)
         self.defineCredentials(**d_pLog)
         self.request = str_logExec
@@ -344,9 +382,11 @@ class c_db_withLog(c_db_dfCredentials):
         self.executeReq()
         self.commit()
         # Go back to the original Credentials
-        d_pDefault = dict(server=self.__server_default, database=self.__database_default, uid=self.__uid_default, pwd=self.__pwd_default)
+        d_pDefault = dict(server=self.__server_default, database=self.__database_default, uid=self.__uid_default,
+                          pwd=self.__pwd_default)
         self.defineCredentials(**d_pDefault)
-    def getDfLog(self, str_logExec = ''):
+
+    def getDfLog(self, str_logExec=''):
         d_pLog = dict(server=self.server_Log, database=self.db_Log, uid=self.__uid_Log, pwd=self.__pwd_Log)
         self.defineCredentials(**d_pLog)
         self.request = str_logExec
@@ -354,29 +394,49 @@ class c_db_withLog(c_db_dfCredentials):
         self.getDataframe()
         self.commit()
         # Go back to the original Credentials
-        d_pDefault = dict(server=self.__server_default, database=self.__database_default, uid=self.__uid_default, pwd=self.__pwd_default)
+        d_pDefault = dict(server=self.__server_default, database=self.__database_default, uid=self.__uid_default,
+                          pwd=self.__pwd_default)
         self.defineCredentials(**d_pDefault)
 
 
-
-#---------------------------------------------------------------
-#----- Function to launch the Class ----------------------------
-#---------------------------------------------------------------
+# ---------------------------------------------------------------
+# ----- Function to launch the Class ----------------------------
+# ---------------------------------------------------------------
 def db_defineInstance():
     dbServer = c_db_withLog()
     return dbServer
 
+
 def db_DefineConnectCursor(str_req, str_server=None, str_database=None, df_UID=None, t_logId=None):
     try:
         dbServer = c_db_withLog()
+        # --------------------------------
+        # Should not be used - SHould be initiliazed by the App at the launch
         if not df_UID is None:
             dbServer.dataframeCredentials(df_UID)
         if not t_logId is None:
             dbServer.define_Log_Cred(str_serverForLog=t_logId(0), str_databaseForLog=t_logId(1))
+        # --------------------------------
+
+        dbServer.bl_useLog = False
+        # Change the Server / database
         if not str_server is None:
             dbServer.change_server(str_server)
+
+        # --------------------------------
+        # Exception: if Database for Log is called, we need to use the server for log (LEGACY)
+        #   And not change the default Database - because it cannot connect with Server
         if not str_database is None:
-            dbServer.change_database(str_database)
+            if dbServer.db_Log.lower() == str_database.lower():
+                print('  (**) Database will be |{}| only for this reqq  \n'.format(dbServer.db_Log))
+                dbServer.bl_useLog = True
+            elif '{}.'.format(dbServer.db_Log).lower() in str_req.lower():
+                print('  (**) Database will be |{}| for this request: \n   **{}**'.format(dbServer.db_Log, str_req))
+                dbServer.bl_useLog = True
+            else:
+                dbServer.change_database(str_database)
+        # --------------------------------
+
         dbServer.request = str_req
         dbServer.connect()
     except Exception as err:
@@ -384,13 +444,13 @@ def db_DefineConnectCursor(str_req, str_server=None, str_database=None, df_UID=N
         raise
     return True
 
-def db_SelectReq(str_req, str_server=None, str_database=None, df_UID=None, t_logId=None, bl_AlertIfEmptyReq = True):
+
+def db_SelectReq(str_req, str_server=None, str_database=None, df_UID=None, t_logId=None, bl_AlertIfEmptyReq=True):
     try:
         dbServer = c_db_withLog()
         db_DefineConnectCursor(str_req, str_server, str_database, df_UID, t_logId)
-        dbServer.defineCredentials(bl_AlertIfEmptyReq = bl_AlertIfEmptyReq)
-        if '{}.'.format(dbServer.db_Log).lower() in str_req.lower():
-            print('  (**) Database will be {} for this request: \n   **|{}|** \n'.format(dbServer.db_Log, str_req))
+        dbServer.defineCredentials(bl_AlertIfEmptyReq=bl_AlertIfEmptyReq)
+        if dbServer.bl_useLog is True:
             dbServer.getDfLog(str_req)
         else:
             dbServer.getDataframe()
@@ -400,6 +460,22 @@ def db_SelectReq(str_req, str_server=None, str_database=None, df_UID=None, t_log
         raise
     return dbServer.df_result
 
+
+def db_EXEC(str_req, str_server=None, str_database=None, df_UID=None, t_logId=None):
+    try:
+        dbServer = c_db_withLog()
+        db_DefineConnectCursor(str_req, str_server, str_database, df_UID, t_logId)
+        if dbServer.bl_useLog is True:
+            dbServer.executeLog(str_req)
+        else:
+            dbServer.executeReq()
+            dbServer.commit()
+    except Exception as err:
+        print('  ERROR in db_EXEC: {}'.format(err))
+        raise
+    return True
+
+
 def db_MultipleReq(str_req, str_server=None, str_database=None, df_UID=None, t_logId=None):
     try:
         dbServer = c_db_withLog()
@@ -412,50 +488,36 @@ def db_MultipleReq(str_req, str_server=None, str_database=None, df_UID=None, t_l
         raise
     return dbServer.df_result
 
-def db_EXEC(str_req, str_server=None, str_database=None, df_UID = None, t_logId = None):
-    try:
-        dbServer = c_db_withLog()
-        db_DefineConnectCursor(str_req, str_server, str_database, df_UID, t_logId)
-        if '{}.'.format(dbServer.db_Log).lower() in str_req.lower():
-            print('  (**) Database will be {} for this request: \n   **{}**'.format(dbServer.db_Log, str_req))
-            dbServer.executeLog(str_req)
-        else:
-            dbServer.executeReq()
-            dbServer.commit()
-    except Exception as err:
-        print('  ERROR in db_EXEC: {}'.format(err))
-        raise
-    return True
 
-
-
-
-#-------------------------------------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------------------------------------
 # Read DB, Save into CSV and read CSV after that (avoid connexion to the same DB several times)
-#-------------------------------------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------------------------------------
 @oth.dec_singletonsClass
 class c_FolderCsv:
     def __init__(self):
         self.__folder = None
+
     @property
     def folder(self):
         return self.__folder
+
     @folder.setter
     def folder(self, str_folder=None):
         if not str_folder is None:
             self.__folder = str_folder
 
-def fDf_readDB_orReadCsv(str_req, str_csvName, int_dayToKeep, str_folderCsv = None, bl_AlertIfEmptyReq = True):
+
+def fDf_readDB_orReadCsv(str_req, str_csvName, int_dayToKeep, str_folderCsv=None, bl_AlertIfEmptyReq=True):
     # Manage so you dont need to tell everytime the folder we are in
     if (str_folderCsv is None) or (str_folderCsv == ''):
-        FolderCsv =     c_FolderCsv()
+        FolderCsv = c_FolderCsv()
         str_folderCsv = FolderCsv.folder
     else:
-        FolderCsv =         c_FolderCsv()
-        FolderCsv.folder =  str_folderCsv
+        FolderCsv = c_FolderCsv()
+        FolderCsv.folder = str_folderCsv
 
     # Boolean
-    bl_fileExist =  False
+    bl_fileExist = False
     bl_fileTooOld = False
 
     # ----- Get the Path -----
@@ -489,11 +551,15 @@ def fDf_readDB_orReadCsv(str_req, str_csvName, int_dayToKeep, str_folderCsv = No
 
     # ----- Delete the file : too old -----
     if bl_fileExist is True and bl_fileTooOld is True:
-        try:    fl.del_fichier_ifOldEnought(str_Path, '', int_dayToKeep)
-        except: print(' Warning in db.fDf_readDB_orReadCsv: (os.delete) \n |{}| \n'.format(str_Path))
+        try:
+            fl.del_fichier_ifOldEnought(str_Path, '', int_dayToKeep)
+        except:
+            print(' Warning in db.fDf_readDB_orReadCsv: (os.delete) \n |{}| \n'.format(str_Path))
 
     # ----- Save the request on CSV -----
-    try:    df_return.to_csv(str_Path, index=False, header=True)
-    except: print(' Warning in db.fDf_readDB_orReadCsv: (df_return.to_csv) \n |{}| \n'.format(str_Path))
+    try:
+        df_return.to_csv(str_Path, index=False, header=True)
+    except:
+        print(' Warning in db.fDf_readDB_orReadCsv: (df_return.to_csv) \n |{}| \n'.format(str_Path))
 
     return df_return
